@@ -3,37 +3,27 @@ from __future__ import annotations
 import operator
 from functools import reduce
 from itertools import chain
-from typing import TYPE_CHECKING
-from typing import Any
-from typing import Iterable
-from typing import Sequence
+from typing import TYPE_CHECKING, Iterable, Sequence
 
 import duckdb
-from duckdb import CoalesceOperator
-from duckdb import FunctionExpression
-from duckdb.typing import BIGINT
-from duckdb.typing import VARCHAR
+from duckdb import CoalesceOperator, Expression, FunctionExpression
+from duckdb.typing import BIGINT, VARCHAR
 
-from narwhals._compliant import CompliantThen
-from narwhals._compliant import LazyNamespace
-from narwhals._compliant import LazyWhen
+from narwhals._compliant import LazyNamespace, LazyThen, LazyWhen
 from narwhals._duckdb.dataframe import DuckDBLazyFrame
 from narwhals._duckdb.expr import DuckDBExpr
 from narwhals._duckdb.selectors import DuckDBSelectorNamespace
-from narwhals._duckdb.utils import concat_str
-from narwhals._duckdb.utils import lit
-from narwhals._duckdb.utils import narwhals_to_native_dtype
-from narwhals._duckdb.utils import when
-from narwhals._expression_parsing import combine_alias_output_names
-from narwhals._expression_parsing import combine_evaluate_output_names
-from narwhals.utils import Implementation
+from narwhals._duckdb.utils import concat_str, lit, narwhals_to_native_dtype, when
+from narwhals._expression_parsing import (
+    combine_alias_output_names,
+    combine_evaluate_output_names,
+)
+from narwhals._utils import Implementation
 
 if TYPE_CHECKING:
-    from typing_extensions import Self
-
+    from narwhals._utils import Version
     from narwhals.dtypes import DType
-    from narwhals.typing import ConcatMethod
-    from narwhals.utils import Version
+    from narwhals.typing import ConcatMethod, NonNestedLiteral
 
 
 class DuckDBNamespace(
@@ -41,14 +31,12 @@ class DuckDBNamespace(
 ):
     _implementation: Implementation = Implementation.DUCKDB
 
-    def __init__(
-        self: Self, *, backend_version: tuple[int, ...], version: Version
-    ) -> None:
+    def __init__(self, *, backend_version: tuple[int, ...], version: Version) -> None:
         self._backend_version = backend_version
         self._version = version
 
     @property
-    def selectors(self: Self) -> DuckDBSelectorNamespace:
+    def selectors(self) -> DuckDBSelectorNamespace:
         return DuckDBSelectorNamespace.from_namespace(self)
 
     @property
@@ -60,7 +48,7 @@ class DuckDBNamespace(
         return DuckDBLazyFrame
 
     def concat(
-        self: Self, items: Iterable[DuckDBLazyFrame], *, how: ConcatMethod
+        self, items: Iterable[DuckDBLazyFrame], *, how: ConcatMethod
     ) -> DuckDBLazyFrame:
         native_items = [item._native_frame for item in items]
         items = list(items)
@@ -73,12 +61,9 @@ class DuckDBNamespace(
         return first._with_native(res)
 
     def concat_str(
-        self: Self,
-        *exprs: DuckDBExpr,
-        separator: str,
-        ignore_nulls: bool,
+        self, *exprs: DuckDBExpr, separator: str, ignore_nulls: bool
     ) -> DuckDBExpr:
-        def func(df: DuckDBLazyFrame) -> list[duckdb.Expression]:
+        def func(df: DuckDBLazyFrame) -> list[Expression]:
             cols = list(chain.from_iterable(expr(df) for expr in exprs))
             if not ignore_nulls:
                 null_mask_result = reduce(operator.or_, (s.isnull() for s in cols))
@@ -104,8 +89,8 @@ class DuckDBNamespace(
             version=self._version,
         )
 
-    def all_horizontal(self: Self, *exprs: DuckDBExpr) -> DuckDBExpr:
-        def func(df: DuckDBLazyFrame) -> list[duckdb.Expression]:
+    def all_horizontal(self, *exprs: DuckDBExpr) -> DuckDBExpr:
+        def func(df: DuckDBLazyFrame) -> list[Expression]:
             cols = (c for _expr in exprs for c in _expr(df))
             return [reduce(operator.and_, cols)]
 
@@ -117,8 +102,8 @@ class DuckDBNamespace(
             version=self._version,
         )
 
-    def any_horizontal(self: Self, *exprs: DuckDBExpr) -> DuckDBExpr:
-        def func(df: DuckDBLazyFrame) -> list[duckdb.Expression]:
+    def any_horizontal(self, *exprs: DuckDBExpr) -> DuckDBExpr:
+        def func(df: DuckDBLazyFrame) -> list[Expression]:
             cols = (c for _expr in exprs for c in _expr(df))
             return [reduce(operator.or_, cols)]
 
@@ -130,8 +115,8 @@ class DuckDBNamespace(
             version=self._version,
         )
 
-    def max_horizontal(self: Self, *exprs: DuckDBExpr) -> DuckDBExpr:
-        def func(df: DuckDBLazyFrame) -> list[duckdb.Expression]:
+    def max_horizontal(self, *exprs: DuckDBExpr) -> DuckDBExpr:
+        def func(df: DuckDBLazyFrame) -> list[Expression]:
             cols = (c for _expr in exprs for c in _expr(df))
             return [FunctionExpression("greatest", *cols)]
 
@@ -143,8 +128,8 @@ class DuckDBNamespace(
             version=self._version,
         )
 
-    def min_horizontal(self: Self, *exprs: DuckDBExpr) -> DuckDBExpr:
-        def func(df: DuckDBLazyFrame) -> list[duckdb.Expression]:
+    def min_horizontal(self, *exprs: DuckDBExpr) -> DuckDBExpr:
+        def func(df: DuckDBLazyFrame) -> list[Expression]:
             cols = (c for _expr in exprs for c in _expr(df))
             return [FunctionExpression("least", *cols)]
 
@@ -156,8 +141,8 @@ class DuckDBNamespace(
             version=self._version,
         )
 
-    def sum_horizontal(self: Self, *exprs: DuckDBExpr) -> DuckDBExpr:
-        def func(df: DuckDBLazyFrame) -> list[duckdb.Expression]:
+    def sum_horizontal(self, *exprs: DuckDBExpr) -> DuckDBExpr:
+        def func(df: DuckDBLazyFrame) -> list[Expression]:
             cols = (CoalesceOperator(col, lit(0)) for _expr in exprs for col in _expr(df))
             return [reduce(operator.add, cols)]
 
@@ -169,8 +154,8 @@ class DuckDBNamespace(
             version=self._version,
         )
 
-    def mean_horizontal(self: Self, *exprs: DuckDBExpr) -> DuckDBExpr:
-        def func(df: DuckDBLazyFrame) -> list[duckdb.Expression]:
+    def mean_horizontal(self, *exprs: DuckDBExpr) -> DuckDBExpr:
+        def func(df: DuckDBLazyFrame) -> list[Expression]:
             cols = [c for _expr in exprs for c in _expr(df)]
             return [
                 (
@@ -187,11 +172,13 @@ class DuckDBNamespace(
             version=self._version,
         )
 
-    def when(self: Self, predicate: DuckDBExpr) -> DuckDBWhen:
+    def when(self, predicate: DuckDBExpr) -> DuckDBWhen:
         return DuckDBWhen.from_expr(predicate, context=self)
 
-    def lit(self: Self, value: Any, dtype: DType | type[DType] | None) -> DuckDBExpr:
-        def func(_df: DuckDBLazyFrame) -> list[duckdb.Expression]:
+    def lit(
+        self, value: NonNestedLiteral, dtype: DType | type[DType] | None
+    ) -> DuckDBExpr:
+        def func(_df: DuckDBLazyFrame) -> list[Expression]:
             if dtype is not None:
                 return [
                     lit(value).cast(
@@ -208,8 +195,8 @@ class DuckDBNamespace(
             version=self._version,
         )
 
-    def len(self: Self) -> DuckDBExpr:
-        def func(_df: DuckDBLazyFrame) -> list[duckdb.Expression]:
+    def len(self) -> DuckDBExpr:
+        def func(_df: DuckDBLazyFrame) -> list[Expression]:
             return [FunctionExpression("count")]
 
         return self._expr(
@@ -221,17 +208,15 @@ class DuckDBNamespace(
         )
 
 
-class DuckDBWhen(LazyWhen["DuckDBLazyFrame", duckdb.Expression, DuckDBExpr]):
+class DuckDBWhen(LazyWhen["DuckDBLazyFrame", Expression, DuckDBExpr]):
     @property
     def _then(self) -> type[DuckDBThen]:
         return DuckDBThen
 
-    def __call__(self: Self, df: DuckDBLazyFrame) -> Sequence[duckdb.Expression]:
+    def __call__(self, df: DuckDBLazyFrame) -> Sequence[Expression]:
         self.when = when
         self.lit = lit
         return super().__call__(df)
 
 
-class DuckDBThen(
-    CompliantThen["DuckDBLazyFrame", duckdb.Expression, DuckDBExpr], DuckDBExpr
-): ...
+class DuckDBThen(LazyThen["DuckDBLazyFrame", Expression, DuckDBExpr], DuckDBExpr): ...
