@@ -1,30 +1,25 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING
-from typing import Any
-from typing import Callable
-from typing import Literal
-from typing import Mapping
-from typing import Sequence
+from typing import TYPE_CHECKING, Any, Callable, Literal, Mapping, Sequence
 
 import polars as pl
 
 from narwhals._duration import parse_interval_string
-from narwhals._polars.utils import extract_args_kwargs
-from narwhals._polars.utils import extract_native
-from narwhals._polars.utils import narwhals_to_native_dtype
-from narwhals.utils import Implementation
-from narwhals.utils import requires
+from narwhals._polars.utils import (
+    extract_args_kwargs,
+    extract_native,
+    narwhals_to_native_dtype,
+)
+from narwhals._utils import Implementation, requires
 
 if TYPE_CHECKING:
     from typing_extensions import Self
 
-    from narwhals._expression_parsing import ExprKind
-    from narwhals._expression_parsing import ExprMetadata
+    from narwhals._expression_parsing import ExprKind, ExprMetadata
     from narwhals._polars.dataframe import Method
     from narwhals._polars.namespace import PolarsNamespace
+    from narwhals._utils import Version
     from narwhals.dtypes import DType
-    from narwhals.utils import Version
 
 
 class PolarsExpr:
@@ -101,14 +96,16 @@ class PolarsExpr:
             native = pl.when(self.native.is_not_null()).then(self.native.is_nan())
         return self._with_native(native)
 
-    def over(self, partition_by: Sequence[str], order_by: Sequence[str] | None) -> Self:
+    def over(self, partition_by: Sequence[str], order_by: Sequence[str]) -> Self:
         if self._backend_version < (1, 9):
             if order_by:
                 msg = "`order_by` in Polars requires version 1.10 or greater"
                 raise NotImplementedError(msg)
             native = self.native.over(partition_by or pl.lit(1))
         else:
-            native = self.native.over(partition_by or pl.lit(1), order_by=order_by)
+            native = self.native.over(
+                partition_by or pl.lit(1), order_by=order_by or None
+            )
         return self._with_native(native)
 
     @requires.backend_version((1,))
