@@ -64,8 +64,10 @@ def test_diff_lazy_grouped(
         pytest.skip()
     if "pandas" in str(constructor) and PANDAS_VERSION < (1, 1):
         pytest.skip()
-    if any(x in str(constructor) for x in ("dask", "pyarrow_table")):
+    if any(x in str(constructor) for x in ("dask", "pyarrow_table", "cudf")):
         # https://github.com/dask/dask/issues/11806
+        # https://github.com/rapidsai/cudf/issues/18160
+        # wooah their issue numbers use exactly the same digits but in a different order
         request.applymarker(pytest.mark.xfail)
     data = {"i": [0, 1, 2, 3, 4], "b": [1, 1, 1, 2, 2], "c": [5, 4, 3, 2, 1]}
     df = nw.from_native(constructor(data))
