@@ -46,8 +46,9 @@ def test_shift_lazy(constructor: Constructor) -> None:
 def test_shift_lazy_grouped(
     constructor: Constructor, request: pytest.FixtureRequest
 ) -> None:
-    if any(x in str(constructor) for x in ("dask", "pyarrow_table")):
+    if any(x in str(constructor) for x in ("dask", "pyarrow_table", "cudf")):
         # https://github.com/dask/dask/issues/11806
+        # https://github.com/rapidsai/cudf/issues/18159
         request.applymarker(pytest.mark.xfail)
     if "polars" in str(constructor) and POLARS_VERSION < (1, 10):
         pytest.skip()
